/*
 * Copyright 2023, Beijing ESWIN Computing Technology Co., Ltd.. All rights reserved.
 * SPDX-License-Identifier: GPL-2.0-only
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __ESWIN_NUMA_H__
#define __ESWIN_NUMA_H__

#include <essdk/es_type.h>
#include <essdk/es_errno.h>
#include <essdk/es_common.h>

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

#define ES_NUMA_ERROR_BASE      ES_DEF_ERR(ES_ID_NUMA, EN_ERR_LEVEL_ERROR, 0)
#define ES_NUMA_ERROR_NUM(X)    (ES_NUMA_ERROR_BASE + X)

#define ES_NUMA_ERROR_ILLEGAL_PARAM            ES_NUMA_ERROR_NUM(EN_ERR_ILLEGAL_PARAM)
#define ES_NUMA_ERROR_NULL_PTR                 ES_NUMA_ERROR_NUM(EN_ERR_NULL_PTR)
#define ES_NUMA_ERROR_NOT_SUPPORT              ES_NUMA_ERROR_NUM(EN_ERR_NOT_SUPPORT)

/*
   Check whether numa is enabled in the system
   @return:
       ES_TRUE : enabled
       ES_FALSE : not enabled
*/
ES_S32 ES_NUMA_Avaiable();

/*
   Get the numa node of the current thread
   @return:
      -ES_NUMA_ERROR_NOT_SUPPORT : operation support
*/
ES_S32 ES_NUMA_GetCurrentNode(void);

/*
    Check weather the current thread is bound to one numa node
    @return:
         ES_FALSE: not bind
         ES_TRUE: bind
*/
ES_S32 ES_NUMA_IsBindNode(void);

/*
    Binds the current task and its children to one node specified by node_idx.
    They will only run on the CPUs of the specified node and allocate memory from it.
    @node_idx : the node id to bind, begin from 0
    @return:
          ES_SUCCESS when sucess
          < 0 when faild
*/
ES_S32 ES_NUMA_BindNode(int node_idx);

/*
    Get the numa node cnt of the current hw system
    @return :
     -ES_NUMA_ERROR_NOT_SUPPORT when failed
*/
ES_S32 ES_NUMA_GetNodeCnt(void);

/*
    Get the numa node of the memory refered by addr.
    The memory could be the noraml cpu memory(eg, alloced by
    malloc/numa_alloc), or the dma memory such as alloced
    by ES_SYS_MemAlloc/ES_VB_GetBlock/ES_VB_GetModBlock
    @return :
       < 0 when faild
       > 0 the numd node id
*/
ES_S32 ES_NUMA_GetMemoryNodeByAddr(void *addr);

/*
    Get the numa node of the memory refered by MemFd.
    The memory should be the dma memory such as alloced
    by ES_SYS_MemAlloc/ES_VB_GetBlock/ES_VB_GetModBlock)
    @return :
       < 0 when faild
       > 0 the numd node id
*/
ES_S32 ES_NUMA_GetMemoryNodeByMemFd(int MemFd);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif
